;;########################################################################
;; unimob1.lsp
;; contains code to implement univariate analysis model object
;; Copyright (c) 1995-7 by Forrest W. Young
;;########################################################################

;3 ViSta system methods (continued)

(defmeth univar-model-object-proto :report (&key (dialog nil))
  (let* ((var (send self :var))
         (var1 (send self :var1))
         (var2 (send self :var2))
         (var1-label (send self :var1-label))
         (var2-label (send self :var2-label))
         (n-ways (send self :n-ways))
         (mu  (send self :mu))
         (direction (send self :direction))
         (ci-level  (send self :ci-level))
         (ci-limits (send self :t-ci-limits))
         (df    (send self :t-df))
         (t-val (send self :t-value))
         (p-lev (send self :t-p-level))
         (n1 (length var1))
         (mean1  (mean var1))
         (stdev1 (standard-deviation var1))
         (sterr (send self :std-err))
         (n2 nil)
         (mean2 nil)
         (stdev2 nil)
         (meandif nil)
         (stdevdif nil)
         (sterrdif nil)
         (pooled-variance nil)
         (compare "one")
         (sign "NE")
         (paired-st "Single Sample")
         (zt "T")
         (sigma (send self :sigma))
         (w nil)
         (mus "Mu")
         (nonpar-st "Wilcoxon's Signed Rank Test for")
         (nonpar-st "WILCOXON'S SIGNED RANK TEST - ")
         (nonpar-au "Wilcoxon's   T")
         )
    (when var2
          (setf n2 (length var2))
          (setf mean2  (mean var2))
          (setf stdev2 (standard-deviation var2)))
    (cond
      ((= direction 0)
       (setf compare "two"))
      ((> direction 0) (setf sign "GT"))
      (t (setf sign "LT")))
    (when (send self :sigma) (setf zt "Z"))
    (when (send self :paired) 
          (setf meandif  (mean var))
          (setf stdevdif (standard-deviation var))
          (setf sterrdif (/ stdevdif (sqrt (1+ df))))
          (setf paired-st "Two Paired (Dependent) Samples"))
    (when (> n-ways 0) 
          (setf paired-st "Two Independent Samples")
          (setf nonpar-st "MANN-WHITNEY TEST - ")
          (setf nonpar-au "Mann-Whitney U")
          (setf mus "Mu1 - Mu2")
          (setf sterrdif (send self :std-err-diff))
          (setf pooled-variance (send self :pooled-variance))
          )
    (setf w (report-header (send self :title)))
    (display-string (format nil "Univariate Analysis of the ~a Data."
                            (send (send self :data-object) :name)) w)
    (display-string (format nil "~%Sample 1: ~a" var1-label) w)
    (when var2-label
          (display-string (format nil "~%Sample 2: ~a" var2-label) w))
    (display-string (format nil "~2%~a-TEST - ~a:" zt paired-st) w)
    (display-string 
     (format nil "~%Null hypothesis:        ~a EQ ~7,3f" mus mu) w)
    (display-string
     (format nil "~%Alternative hypothesis: ~a ~a ~7,3f (~a tailed)" 
             mus sign mu compare) w)
    (when sigma (display-string 
       (format nil "~%Known Population Standard Deviation:~7,3f" sigma) w))
    (display-string (format nil "~2%Sample Statistics:") w)
    (display-string (format nil "~%Sample 1:") w)
    (display-string (format nil "  N = ~7d Mean = ~8,3f StDev = ~7,3f " 
                            n1 mean1 stdev1) w)
    (when (not var2)
          (display-string (format nil " StErr = ~7,3f" (send self :std-err))
                          w))
    (when var2 
          (display-string (format nil "~%Sample 2:") w)
          (display-string (format nil "  N = ~7d Mean = ~8,3f StDev = ~7,3f " 
                            n2 mean2 stdev2) w))
    (when (send self :paired)
          (display-string (format nil "~%Sample Differences:    Mean = ~8,3f StDev =~8,3f StErr =~8,3f" meandif stdevdif sterrdif) w))
    (when (> n-ways 0)
          (display-string 
           (format nil "~%Pooled Variance =~8,3f Std. Err. =~8,3f" 
                   pooled-variance sterrdif) w))
    (display-string (format nil "~%~a-Test:  " zt) w)
    (display-string (format nil "  ~a = ~7,3f " zt t-val) w)
    (when (not sigma) 
          (display-string (format nil "df   = ~7f " (round df)) w))
    (display-string (format nil "p = ~6,4f" p-lev) w)
    (display-string
     (format nil "~2%The ~2d% confidence interval is: (~8,3f,~8,3f)~%" 
             (round (* 100 ci-level)) (first ci-limits) 
             (second ci-limits)) w)
    (when (send self :nonpar)
          (display-string
           (format nil "~%~a ~a:~%Null Hypothesis:        Both Populations Identically Distributed " nonpar-st paired-st) w)
          (display-string
           (format nil "~%Alternative Hypothesis: Both Populations Differently Distributed (two tailed)") w)
          (when (> n-ways 0) (display-string
                (format nil "~%Mann-Whitney Sum of Ranks for each group = ~8,3f" 
                        (send self :mann-whitney-sum)) w))
          (display-string
           (format nil "~%~a =~5d~%Approximate  Z = ~8,3f   p = ~6,4f~%"
                   nonpar-au (send self :nonpar) (send self :nonpar-z)
                   (send self :nonpar-p)) w))
    )
  t)

(defmeth univar-model-object-proto :save-model-template (data-object)
"Args: (data-object)
DATA-OBJECT is the object-identification information of a data object. 
The method contains a template for saving the model-object." 
  (let* ((var1-label (send self :var1-label))
         (var2-label (send self :var2-label))
         )
    (cond
      ((= 1 (send self :n-ways))
       `(univariate-analysis    
         :title      ,(send self :title)
         :name       ,(send self :name) 
         :dialog      nil
         :variable   ,(first (send self :variables))
         :mu         ,(send self :mu)
         :sigma      ,(send self :sigma)
         :direction  ,(send self :direction)
         :ci-level   ,(send self :ci-level)
         :data (data ,(send data-object :name)
                     :title      ,(send data-object :title)
                     :variables ',(send self :variables)
                     :types     ',(send self :types)
                     :labels    ',(send self :labels)
                     :ways      ',(send data-object :ways)
                     :classes   ',(send data-object :classes)
                     :data      ',(send self :data))))
      ((not (send self :var2-label))
       `(univariate-analysis    
         :title      ,(send self :title)
         :name       ,(send self :name) 
         :dialog      nil
         :variable   ,var1-label
         :mu         ,(send self :mu)
         :sigma      ,(send self :sigma)
         :direction  ,(send self :direction)
         :ci-level   ,(send self :ci-level)
         :data (data ,(send data-object :name)
                     :title      ,(send data-object :title)
                     :variables ',(send self :variables)
                     :types     ',(send self :types)
                     :labels    ',(send self :labels)
                     :data      ',(send self :data))))
      ((send self :var2-label)
       `(univariate-analysis    
        :title      ,(send self :title)
        :name       ,(send self :name) 
        :dialog      nil
        :variable    (list ,var1-label ,var2-label)
        :mu         ,(send self :mu)
        :sigma      ,(send self :sigma)
        :direction  ,(send self :direction)
        :ci-level   ,(send self :ci-level)
        :data (data ,(send data-object :name)
                    :title      ,(send data-object :title)
                    :variables ',(send self :variables)
                    :types     ',(send self :types)
                    :labels    ',(send self :labels)
                    :data      ',(send self :data))))
      )
    ))

(defmeth univar-model-object-proto :create-data (&rest args)
  (error-message "Not available for Univariate Analysis."))

(defmeth univar-model-object-proto :visualize ()
  (if (send self :var2) 
      (send self :visualize-model-2)
      (send self :visualize-model-1)))

;one sample - two side-by-side plots
 
(defmeth univar-model-object-proto :visualize-model-1 ()
  (if (not (eq current-object self)) (setcm self))
  (let* ((var (send self :var))
         (var-label (send self :var1-label))
         (obs-label (send self :labels))
         (sp (spread-plot (matrix '(1 2) 
               (list
                (boxplot var :diamonds t :variable-labels (list var-label)
                         :point-labels obs-label :show nil)
                (quantile-plot var :reg-line t :variable-label var-label
                          :point-labels obs-label :show nil)))
              :model self))
         (bp (aref (send sp :plot-matrix) 0 0))
         )
    (send bp :new-menu "BoxPlot" 
          :items '(SHOWING-LABELS MOUSE RESIZE-BRUSH DASH ERASE-SELECTION 
                   FOCUS-ON-SELECTION SHOW-ALL SYMBOL COLOR))
    (send bp :showing-labels t)
    sp))

;two samples - four plots, two up, two down
 
(defmeth univar-model-object-proto :visualize-model-2 () 
  (if (not (eq current-object self)) (setcm self))
  (let* ((var1 (send self :var1))
         (var2 (send self :var2))
         (var1-label (send self :var1-label))
         (var2-label (send self :var2-label))
         (point-labels (combine (send self :labels) (send self :labels)))
         (sp (spread-plot 
              (matrix  '(2 2) (list
                (boxplot (list var1 var2) 
                         :variable-labels (list var1-label var2-label)
                         :point-labels point-labels
                         :show nil) 
                (quantile-plot var1 :reg-line t :variable-label var1-label
                               :show nil)
                (quantile-quantile-plot var1 var2 :reg-line t 
                         :variable-labels (list var1-label var2-label)
                                        :show nil)
                (quantile-plot var2 :reg-line t :variable-label var2-label
                               :show nil)))
              :model self))
         (bp  (aref (send sp :plot-matrix) 0 0))
         )
    (send bp :new-menu "BoxPlot" 
          :items '(SHOWING-LABELS MOUSE RESIZE-BRUSH DASH ERASE-SELECTION 
                   FOCUS-ON-SELECTION SHOW-ALL SYMBOL COLOR))
    (send bp :showing-labels t)
    sp))

;4 Object Specific Methods

(defmeth univar-model-object-proto :compute-t-or-z-value ()
"Args: none
Computes t-test for one sample or two paired samples"
  (let ((var (send self :var))
        (mu  (send self :mu))
        (s   (send self :sigma)))
    (when (not s) (setf s (standard-deviation var)))
    (/ (- (mean var) mu) (/ s (sqrt (length var))))))

(defmeth univar-model-object-proto :compute-independent-t ()
"Args: none
Computes t-test for independent samples"
  (let* ((var (send self :var))
         (mu  (send self :mu))
         (mean-dif (apply #'- (mapcar #'mean var)))
         (pooled-var (/ (apply #'sum  (* (- (mapcar #'length var) 1) 
                                         (mapcar #'variance var)))
                        (sum (- (mapcar #'length var) 1))))
         (std-err (sqrt (* pooled-var (sum (/ (mapcar #'length var)))))))
    (send self :pooled-variance pooled-var)
    (send self :std-err-diff std-err)
    (break)
    (/ mean-dif std-err)))

(defmeth univar-model-object-proto :compute-independent-t ()
"Args: none
Computes t-test for independent samples"
  (let* ((var (send self :var))
         (mu  (send self :mu))
         (mean-dif (apply #'- (mapcar #'mean var)))
         (pooled-var (sum (/ (mapcar (function variance) var) 
                             (mapcar (function length) var))))
         (std-err (sqrt pooled-var)))
    (send self :pooled-variance pooled-var)
    (send self :std-err-diff std-err)
    (/ mean-dif std-err)))

(defmeth univar-model-object-proto :compute-t-or-z-p-level ()
"Args: none
Computes significance of t or z statistic"
  (let ((t-value (send self :t-value))
        (df (send self :t-df))
        (direction (send self :direction)))
    (when (send self :sigma) (setf df 1000))
    (cond 
      ((not direction) (* 2 (- 1 (t-cdf (abs t-value) df))))
      ((= direction 0) (* 2 (- 1 (t-cdf (abs t-value) df))))
      ((> direction 0) (- 1 (t-cdf t-value df)))
      ((< direction 0) (t-cdf t-value df)))))

(defmeth univar-model-object-proto :compute-t-or-z-ci-limits ()
"Args: none
Computes confidence limits based on t or z statistic"
  (let* ((var   (send self :var))
         (level (send self :ci-level))
         (mean  (mean var))
         (n     (length var))
         (sterr (/ (standard-deviation var) (sqrt n)))
         (df    (send self :t-df))
         (t-crit nil)
         (width nil))
    (when (send self :sigma) (setf df 1000))
    (when (> (send self :n-ways) 0) 
          (setf mean (apply #'- (mapcar #'mean var)))
          (setf df (1- (min (mapcar #'length var))))
          (setf sterr (send self :std-err-diff)))
    (setf t-crit (t-quant (- 1 (/ (- 1 level) 2)) df))
    (send self :std-err sterr)
    (setf width (* t-crit sterr))
    (list (- mean width) (+ mean width))))

(defmeth univar-model-object-proto :compute-wilcoxon () 
"Args: none
Computes the wilcoxon Signed-Rank test for matched samples."
  (let* ((sign-dif  (remove 0 (send self :var)))
         (n (length sign-dif))
         (ranks nil)
         (WT nil))
    (if (= n (length (remove-duplicates sign-dif)))
        (setf ranks (1+ (rank (abs sign-dif))))
        (setf ranks (1+ (rank-with-ties (abs sign-dif)))))
    (if (< (length (which (< sign-dif 0)))
           (length (which (> sign-dif 0))))
        (setf WT (sum (select ranks (which (< sign-dif 0)))))
        (setf WT (sum (select ranks (which (> sign-dif 0))))))
    (when (numberp wt)
          (send self :nonpar wt)
          (send self :nonpar-z
                (/ (- wt (/ (* n (1+ n)) 4)) 
                   (sqrt (/ (* n (1+ n) (1+ (* 2 n))) 24))))
          (send self :nonpar-p
                (* 2 (- 1 (normal-cdf (abs (send self :nonpar-z)) )))))
    t))

(defmeth univar-model-object-proto :compute-mann-whitney ()
  (let* ((vars (send self :var))
         (ranks (1+ (rank-with-ties (combine vars))))
         (nandm (mapcar #'length vars))
         (m (first nandm))
         (n (second nandm))
         (mn1 (+ m n 1))
         (wx (sum (select ranks (iseq m))))
         (wx2 (sum (select ranks (iseq m (1- (+ m n))))))
         (u (- (+ (* m n) (/ (* m (1+ m)) 2)) wx))
         (z (abs (/ (- wx (/ (* m mn1) 2)) (sqrt (/ (* m n mn1) 12)))))
         (p (* 2 (- 1 (normal-cdf z))))
         ) 
    (when (> u (/ (* m n) 2)) (setf u (- (* m n) u))) 
    (send self :mann-whitney-sum (list wx wx2))
    (send self :nonpar u)
    (send self :nonpar-z z)
    (send self :nonpar-p p)
    t))

;5 Isnew Method

(defmeth univar-model-object-proto :isnew 
  (variable mu sigma direction ci-level &rest args) 
  (let ((n-ways (length (send current-data :ways))))
    (cond
      ((< (send current-data :active-nvar '(numeric)) 1)
       (error-message "There are no active numeric variables in the data.")
       (send *toolbox* :reset-button (first args)))
      ((> n-ways 1) (error-message "Multi-Way table data are not appropriate for Univariate Analysis. Try Analysis of Variance instead.")
       (send *toolbox* :reset-button (first args)))
      (t
       (when (= n-ways 1) 
             (when (> (length (first (send current-data :classes))) 2)
                   (send *toolbox* :reset-button (first args))
                   (error "One-Way table data may only have 2 levels (classes) for Univariate Analysis. Try Analysis of Variance.")))
       (send self :model-abbrev "UNI")
       (send self :var (send current-data :data))
       (send self :mu mu)
       (send self :sigma sigma)
       (send self :direction direction)
       (send self :ci-level ci-level)
       (send self :n-ways n-ways)
       (send create-dataobjects-model-popup-menu-item :enabled nil)
       (send create-dataobjects-model-menu-item :enabled nil)
       (cond 
         ((> n-ways 0)
          (send self :var1-label 
                (first (first (send current-data :classes))))
          (send self :var2-label 
                (second (first (send current-data :classes))))
          (send self :var  (send current-data :data))
          (send self :var1 (first (send self :var)))
          (send self :var2 (second (send self :var))))
         ((stringp variable)
          (send self :var1-label variable)
          (send self :var  (combine (send current-data :variable variable)))
          (send self :var1 (send self :var)) )
         ((and (stringp (first variable)) (stringp (second variable)))
          (send self :paired t)
          (send self :var1-label (first variable))
          (send self :var2-label (second variable))
          (send self :var (- (send current-data :variable (first variable))
                             (send current-data :variable (second variable))))
          (send self :var1 (send current-data :variable (first variable)))
          (send self :var2 (send current-data :variable (second variable))))
         (t (error "Analysis variable(s) not specified correctly.")))
       (apply #'call-next-method args)))))

;6 Constructor Function

(defun univariate-analysis  
  (&key
   (data current-data)
   (title "Univariate Analysis")
   (name (strcat "UNI-" (send current-data :name)))
   (dialog nil)
   (variable (first (send current-data :active-variables '(numeric))))
   (mu 0)
   (sigma nil)
   (direction 0)
   (ci-level .95)
   ) 
"Args:(dialog nil) (data current-data) (title Univariate Analysis)
      (variable (first (send current-data :active-variables '(nmumeric))))
      (mu 0) (sigma nil)  (direction 0)  (ci-level .95) 
ViSta function to perform Univariate Analysis. Data may be either multivariate or table data. 
  Single sample analysis occurs with multivariate data when VARIABLE is a single string naming one variable or when no variable is specified (in which case the first variable in the dataset is used). 
  To perform paired samples analysis, the data must be multivariate and VARIABLE must be a list of two variable name strings.  
  Independent samples analysis occurs when the data are table data and when VARIABLE is a single string naming one variable.  
  With no arguments the first active numeric variable of the current data are analyzed, with the remaining variables being ignored. Keyword arguments are:
:DATA followed by the name of the data to be analyzed (default: current-data);
:TITLE followed by a character string (default: Univariate Analysis);
:DIALOG followed by t (to display parameters dialog box) or nil (default);
:VARIABLE followed by a string which is the name of the variable to be analyzed (default is the first active numeric variable);
:MU followed by a number specifying the population mean under the null hypothesis (default is 0);
:SIGMA followed by a number specifying the population standard deviation, if known (default is nil, implying SIGMA is unknown);
:DIRECTION followed by -1 (negative 1-tailed), 0 (default, two-tailed), or 1 (positive one-tailed) specifying the type of t-test.
:CI-LEVEL followed by a positive number near 1 specifing the confidence interval level (default is .95)."
  (if (not (eq current-object data)) (setcd data)) 
  (send univar-model-object-proto
        :new variable mu sigma direction ci-level 
        8 data title name dialog))

(load (strcat *vista-dir-name* "sprdplt1")) 